@@ -1,8 +1,8 @@
 /* cmdline.c - core analysis suite
  *
  * Copyright (C) 1999, 2000, 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 David Anderson
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 David Anderson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,7 +47,7 @@ static void readline_init(void);
 static struct alias_data alias_head = { 0 }; 
 
 void
-get_command_line(void)
+process_command_line(void)
 {
 	/*
 	 *  Restore normal environment, clearing out any excess baggage
@@ -1056,8 +1056,10 @@ restore_sanity(void)
 	}
 
 	if (pc->flags & TTY) {
-		if ((fd = open("/dev/tty", O_RDONLY)) < 0) 
-			error(FATAL, "/dev/tty: %s\n", strerror(errno));
+		if ((fd = open("/dev/tty", O_RDONLY)) < 0) {
+			console("/dev/tty: %s\n", strerror(errno));
+			clean_exit(1);
+		}
 	        
 	        if (tcsetattr(fd, TCSANOW, &pc->termios_orig) == -1) 
                         error(FATAL, "tcsetattr /dev/tty: %s\n",
@@ -1439,6 +1441,9 @@ alias_init(char *inbuf)
         argcnt = parse_line(buf, args);
         allocate_alias(ALIAS_BUILTIN);
 
+        strcpy(buf, "alias lsmod mod");
+        argcnt = parse_line(buf, args);
+        allocate_alias(ALIAS_BUILTIN);
 }
 
 /*
