@@ -1,14 +1,17 @@
 /* netdump.c 
  *
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 David Anderson
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 David Anderson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Red Hat, Inc. All rights reserved.
  *
- * This software may be freely redistributed under the terms of the
- * GNU General Public License.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
  * Author: David Anderson
  */
@@ -210,6 +213,11 @@ is_netdump(char *file, ulong source_query)
 				goto bailout;
 			break;
 
+		case EM_S390:
+			if (machine_type_mismatch(file, "S390X", NULL,
+			    source_query))
+				goto bailout;
+			break;
 		case EM_386:
 			if (machine_type_mismatch(file, "X86", NULL,
 			    source_query))
@@ -303,7 +311,8 @@ is_netdump(char *file, ulong source_query)
                 	dump_Elf32_Phdr(nd->load32 + i, ELFSTORE+i);
         	offset32 = nd->notes32->p_offset;
                 for (tot = 0; tot < nd->notes32->p_filesz; tot += len) {
-                        len = dump_Elf32_Nhdr(offset32, ELFSTORE);
+                        if (!(len = dump_Elf32_Nhdr(offset32, ELFSTORE)))
+				break;
                         offset32 += len;
                 }
 		break;
@@ -331,7 +340,8 @@ is_netdump(char *file, ulong source_query)
                 	dump_Elf64_Phdr(nd->load64 + i, ELFSTORE+i);
                 offset64 = nd->notes64->p_offset;
                 for (tot = 0; tot < nd->notes64->p_filesz; tot += len) {
-                        len = dump_Elf64_Nhdr(offset64, ELFSTORE);
+                        if (!(len = dump_Elf64_Nhdr(offset64, ELFSTORE)))
+				break;
                         offset64 += len;
                 }
 		break;
@@ -471,7 +481,8 @@ read_netdump(int fd, void *bufptr, int cnt, ulong addr, physaddr_t paddr)
 	case KDUMP_ELF32:
 	case KDUMP_ELF64:
 		if (nd->num_pt_load_segments == 1) {
-			offset = (off_t)paddr + (off_t)nd->header_size;
+			offset = (off_t)paddr + (off_t)nd->header_size -
+				(off_t)nd->pt_load_segments[0].phys_start;
 			break;
 		}
 
@@ -692,6 +703,9 @@ get_netdump_panic_task(void)
                 else
                         note32 = (Elf32_Nhdr *)nd->nt_prstatus;
 
+		if (!note32)
+			goto panic_task_undetermined;
+
 	        len = sizeof(Elf32_Nhdr);
 	        len = roundup(len + note32->n_namesz, 4);
 	        len = roundup(len + note32->n_descsz, 4);
@@ -736,6 +750,9 @@ check_ebp_esp:
                 else
                         note64 = (Elf64_Nhdr *)nd->nt_prstatus;
 
+		if (!note64)
+			goto panic_task_undetermined;
+
 	        len = sizeof(Elf64_Nhdr);
 	        len = roundup(len + note64->n_namesz, 4);
 		user_regs = (char *)((char *)note64 + len +
@@ -978,7 +995,8 @@ netdump_memory_dump(FILE *fp)
 			dump_Elf32_Phdr(nd->load32 + i, ELFREAD);
         	offset32 = nd->notes32->p_offset;
         	for (tot = 0; tot < nd->notes32->p_filesz; tot += len) {
-                	len = dump_Elf32_Nhdr(offset32, ELFREAD);
+                	if (!(len = dump_Elf32_Nhdr(offset32, ELFREAD)))
+				break;
 			offset32 += len;
         	}
 		break;
@@ -991,7 +1009,8 @@ netdump_memory_dump(FILE *fp)
 			dump_Elf64_Phdr(nd->load64 + i, ELFREAD);
         	offset64 = nd->notes64->p_offset;
         	for (tot = 0; tot < nd->notes64->p_filesz; tot += len) {
-                	len = dump_Elf64_Nhdr(offset64, ELFREAD);
+                	if (!(len = dump_Elf64_Nhdr(offset64, ELFREAD)))
+				break;
                 	offset64 += len;
         	}
 		break;
@@ -1298,6 +1317,11 @@ dump_Elf32_Phdr(Elf32_Phdr *prog, int store_pt_load_data)
 	int others;
 	struct pt_load_segment *pls;
 
+        if ((char *)prog > (nd->elf_header + nd->header_size))
+		error(FATAL,
+		    "Elf32_Phdr pointer: %lx  ELF header end: %lx\n\n",
+			(char *)prog, nd->elf_header + nd->header_size);
+
 	if (store_pt_load_data) 
 		pls = &nd->pt_load_segments[store_pt_load_data-1];
 	else
@@ -1387,6 +1411,11 @@ dump_Elf64_Phdr(Elf64_Phdr *prog, int store_pt_load_data)
 	else
 		pls = NULL;
 
+        if ((char *)prog > (nd->elf_header + nd->header_size))
+		error(FATAL,
+		    "Elf64_Phdr pointer: %lx  ELF header end: %lx\n\n",
+			(char *)prog, nd->elf_header + nd->header_size);
+
 	netdump_print("Elf64_Phdr:\n");
 	netdump_print("                 p_type: %lx ", prog->p_type);
 	switch (prog->p_type)
@@ -1564,14 +1593,38 @@ dump_Elf32_Nhdr(Elf32_Off offset, int store)
 	char *ptr;
 	ulong *uptr;
 	int xen_core, vmcoreinfo;
+	uint64_t remaining, notesize;
 
 	note = (Elf32_Nhdr *)((char *)nd->elf32 + offset);
 
-        netdump_print("Elf32_Nhdr:\n");
-        netdump_print("               n_namesz: %ld ", note->n_namesz);
         BZERO(buf, BUFSIZE);
 	xen_core = vmcoreinfo = FALSE;
         ptr = (char *)note + sizeof(Elf32_Nhdr);
+
+	if (ptr > (nd->elf_header + nd->header_size)) {
+		error(WARNING, 
+	    	    "Elf32_Nhdr pointer: %lx ELF header end: %lx\n",
+			(char *)note, nd->elf_header + nd->header_size);
+		return 0;
+	} else
+		remaining = (uint64_t)((nd->elf_header + nd->header_size) - ptr);
+
+	notesize = (uint64_t)note->n_namesz + (uint64_t)note->n_descsz;
+
+	if ((note->n_namesz == 0) || !remaining || (notesize > remaining)) {
+		error(WARNING, 
+		    "possibly corrupt Elf32_Nhdr: "
+		    "n_namesz: %ld n_descsz: %ld n_type: %lx\n%s",
+			note->n_namesz, note->n_descsz, note->n_type,
+			note->n_namesz || note->n_descsz || !remaining ? 
+			"\n" : "");
+		if (note->n_namesz || note->n_descsz || !remaining)
+			return 0;
+	}
+
+        netdump_print("Elf32_Nhdr:\n");
+        netdump_print("               n_namesz: %ld ", note->n_namesz);
+
         BCOPY(ptr, buf, note->n_namesz);
         netdump_print("(\"%s\")\n", buf);
 
@@ -1758,14 +1811,38 @@ dump_Elf64_Nhdr(Elf64_Off offset, int store)
 	int *iptr;
 	ulong *up;
 	int xen_core, vmcoreinfo;
+	uint64_t remaining, notesize;
 
 	note = (Elf64_Nhdr *)((char *)nd->elf64 + offset);
 
-        netdump_print("Elf64_Nhdr:\n");
-        netdump_print("               n_namesz: %ld ", note->n_namesz);
         BZERO(buf, BUFSIZE);
         ptr = (char *)note + sizeof(Elf64_Nhdr);
 	xen_core = vmcoreinfo = FALSE;
+
+	if (ptr > (nd->elf_header + nd->header_size)) {
+		error(WARNING, 
+	    	    "Elf64_Nhdr pointer: %lx  ELF header end: %lx\n\n",
+			(char *)note, nd->elf_header + nd->header_size);
+		return 0;
+	} else
+		remaining = (uint64_t)((nd->elf_header + nd->header_size) - ptr);
+
+	notesize = (uint64_t)note->n_namesz + (uint64_t)note->n_descsz;
+
+	if ((note->n_namesz == 0) || !remaining || (notesize > remaining)) {
+		error(WARNING, 
+		    "possibly corrupt Elf64_Nhdr: "
+		    "n_namesz: %ld n_descsz: %ld n_type: %lx\n%s",
+			note->n_namesz, note->n_descsz, note->n_type,
+			note->n_namesz || note->n_descsz || !remaining ? 
+			"\n" : "");
+		if (note->n_namesz || note->n_descsz || !remaining)
+			return 0;
+	}
+
+        netdump_print("Elf64_Nhdr:\n");
+        netdump_print("               n_namesz: %ld ", note->n_namesz);
+
         BCOPY(ptr, buf, note->n_namesz);
         netdump_print("(\"%s\")\n", buf);
 
@@ -1919,6 +1996,9 @@ dump_Elf64_Nhdr(Elf64_Off offset, int store)
                 break;
 	}
 
+	if (store && machine_type("S390X"))
+		machdep->dumpfile_init(nd->num_prstatus_notes, note);
+
 	uptr = (ulonglong *)(ptr + note->n_namesz);
 
         /*
@@ -2011,6 +2091,9 @@ get_netdump_regs(struct bt_info *bt, ulong *eip, ulong *esp)
 		return get_netdump_regs_x86_64(bt, eip, esp);
 		break;
 
+	case EM_S390:
+		machdep->get_stack_frame(bt, eip, esp);
+		break;
 	default:
 		error(FATAL, 
 		   "support for ELF machine type %d not available\n",
