@@ -1,9 +1,9 @@
 /* s390.c - core analysis suite
  *
  * Copyright (C) 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006 David Anderson
- * Copyright (C) 2002, 2003, 2004, 2005, 2006 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2005, 2006 Michael Holzheu, IBM Corporation
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2009, 2010 David Anderson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2009, 2010 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2005, 2006, 2010 Michael Holzheu, IBM Corporation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@
 #define S390_PTE_INVALID_MASK   0x80000900
 #define S390_PTE_INVALID(x) ((x) & S390_PTE_INVALID_MASK)
 
-#define ASYNC_STACK_SIZE  STACKSIZE() // can be 4096 or 8192
+#define INT_STACK_SIZE    STACKSIZE() // can be 4096 or 8192
 #define KERNEL_STACK_SIZE STACKSIZE() // can be 4096 or 8192
 
 #define LOWCORE_SIZE 4096
@@ -67,6 +67,18 @@ static void s390_dump_line_number(ulong);
 static struct line_number_hook s390_line_number_hooks[];
 static int s390_is_uvaddr(ulong, struct task_context *);
 
+/*
+ * Initialize member offsets
+ */
+static void s390_offsets_init(void)
+{
+	if (MEMBER_EXISTS("_lowcore", "st_status_fixed_logout"))
+		MEMBER_OFFSET_INIT(s390_lowcore_psw_save_area, "_lowcore",
+				   "st_status_fixed_logout");
+	else
+		MEMBER_OFFSET_INIT(s390_lowcore_psw_save_area, "_lowcore",
+				   "psw_save_area");
+}
 
 /*
  *  Do all necessary machine-specific setup here.  This is called several
@@ -132,6 +144,7 @@ s390_init(int when)
 			machdep->hz = HZ;
 		machdep->section_size_bits = _SECTION_SIZE_BITS;
 		machdep->max_physmem_bits = _MAX_PHYSMEM_BITS;
+		s390_offsets_init();
 		break;
 
 	case POST_INIT:
@@ -529,46 +542,17 @@ s390_cpu_of_task(unsigned long task)
 }
 
 /*
- * returns true, if task currently is executed by a cpu
+ * returns true, if task of bt currently is executed by a cpu
  */ 
 static int 
-s390_has_cpu(unsigned long task)
+s390_has_cpu(struct bt_info *bt)
 {
-	if(VALID_MEMBER(task_struct_cpus_runnable)){
-                /* Linux 2.4 */
-                unsigned long cpus_runnable;
-                readmem(task+OFFSET(task_struct_cpus_runnable),KVADDR,
-                        &cpus_runnable,sizeof(cpus_runnable),
-                        "cpus_runnable", FAULT_ON_ERROR);
-                if(cpus_runnable != ~0U)
-                        return TRUE;
-                else
-                        return FALSE;
-        } else {
-		/* Linux 2.6 */
-		unsigned long runqueue_addr, runqueue_offset;
-		unsigned long cpu_offset, per_cpu_offset_addr, running_task;
-		char *runqueue;
-		int cpu;
+	int cpu = bt->tc->processor;
 
-		cpu = s390_cpu_of_task(task);
-		runqueue = GETBUF(SIZE(runqueue));
-
-		runqueue_offset=symbol_value("per_cpu__runqueues");
-		per_cpu_offset_addr=symbol_value("__per_cpu_offset");
-		readmem(per_cpu_offset_addr + cpu * sizeof(long),KVADDR,
-			&cpu_offset, sizeof(long),"per_cpu_offset",
-			FAULT_ON_ERROR);
-		runqueue_addr=runqueue_offset + cpu_offset;
-		readmem(runqueue_addr,KVADDR,runqueue,SIZE(runqueue),
-			"runqueue", FAULT_ON_ERROR);
-		running_task = ULONG(runqueue + OFFSET(runqueue_curr));
-		FREEBUF(runqueue);
-		if(running_task == task)
-			return TRUE;
-		else
-			return FALSE;
-	}
+	if (is_task_active(bt->task) && (kt->cpu_flags[cpu] & ONLINE))
+		return TRUE;
+	else
+		return FALSE;
 }
 
 /*
@@ -586,20 +570,23 @@ s390_get_lowcore(int cpu, char* lowcore)
 		FAULT_ON_ERROR);
 }
 
-/* 
- * read in the async stack
+/*
+ * Read interrupt stack (either "async_stack" or "panic_stack");
  */
-static void
-s390_get_async_stack(char* lowcore, char* async_stack, unsigned long* start, unsigned long* end)
+static void s390_get_int_stack(char *stack_name, char* lc, char* int_stack,
+			       unsigned long* start, unsigned long* end)
 {
-	unsigned long async_stack_ptr;
+	unsigned long stack_addr;
 
-	async_stack_ptr = ULONG(lowcore + 
-				MEMBER_OFFSET("_lowcore","async_stack"));
-	readmem(async_stack_ptr-ASYNC_STACK_SIZE,KVADDR, async_stack, 
-		ASYNC_STACK_SIZE, "async_stack", FAULT_ON_ERROR);
-	*start=async_stack_ptr-ASYNC_STACK_SIZE;
-	*end=async_stack_ptr;
+	if (!MEMBER_EXISTS("_lowcore", stack_name))
+		return;
+	stack_addr = ULONG(lc + MEMBER_OFFSET("_lowcore", stack_name));
+	if (stack_addr == 0)
+		return;
+	readmem(stack_addr - INT_STACK_SIZE, KVADDR, int_stack,
+		INT_STACK_SIZE, stack_name, FAULT_ON_ERROR);
+	*start = stack_addr - INT_STACK_SIZE;
+	*end = stack_addr;
 }
 
 /*
@@ -609,20 +596,27 @@ static void
 s390_back_trace_cmd(struct bt_info *bt)
 {
 	char* stack;
-	char async_stack[ASYNC_STACK_SIZE];
+	char async_stack[INT_STACK_SIZE];
+	char panic_stack[INT_STACK_SIZE];
 	long ksp,backchain,old_backchain;
 	int i=0, r14_offset,bc_offset,r14, skip_first_frame=0;
-	unsigned long async_start,async_end, stack_end, stack_start, stack_base;
+	unsigned long async_start = 0, async_end = 0;
+	unsigned long panic_start = 0, panic_end = 0;
+	unsigned long stack_end, stack_start, stack_base;
+	int cpu = bt->tc->processor;
 
 	if (bt->hp && bt->hp->eip) {
 		error(WARNING,
 		"instruction pointer argument ignored on this architecture!\n");
 	}
-	async_end = async_start = 0;
+	if (is_task_active(bt->task) && !(kt->cpu_flags[cpu] & ONLINE)) {
+		fprintf(fp, " CPU offline\n");
+		return;
+	}
 	ksp = bt->stkptr;
 
 	/* print lowcore and get async stack when task has cpu */
-	if(s390_has_cpu(bt->task)){
+	if(s390_has_cpu(bt)){
 		char lowcore[LOWCORE_SIZE];
 		unsigned long psw_flags;
 		int cpu = s390_cpu_of_task(bt->task);
@@ -632,16 +626,16 @@ s390_back_trace_cmd(struct bt_info *bt)
 			return;
 		}
 		s390_get_lowcore(cpu,lowcore);
-		psw_flags = ULONG(lowcore + MEMBER_OFFSET("_lowcore",
-			    "st_status_fixed_logout"));
+		psw_flags = ULONG(lowcore + OFFSET(s390_lowcore_psw_save_area));
 		if(psw_flags & 0x10000UL){
 				fprintf(fp,"Task runs in userspace\n");
 				s390_print_lowcore(lowcore,bt,0);
 				return;
 		}
-
-		s390_get_async_stack(lowcore,async_stack,&async_start,
-				     &async_end);
+		s390_get_int_stack("async_stack", lowcore, async_stack,
+				   &async_start, &async_end);
+		s390_get_int_stack("panic_stack", lowcore, panic_stack,
+				   &panic_start, &panic_end);
 		s390_print_lowcore(lowcore,bt,1);
 		fprintf(fp,"\n");
 		skip_first_frame=1;
@@ -670,14 +664,18 @@ s390_back_trace_cmd(struct bt_info *bt)
 		unsigned long r14_stack_off;
 		int j;
 
-		/* Find stack: Either async stack or task stack */
+		/* Find stack: Either async, panic stack or task stack */
 		if((backchain > stack_start) && (backchain < stack_end)){
 			stack = bt->stackbuf;
 			stack_base = stack_start;
 		} else if((backchain > async_start) && (backchain < async_end)
-			  && s390_has_cpu(bt->task)){
+			  && s390_has_cpu(bt)){
 			stack = async_stack;
 			stack_base = async_start;
+		} else if((backchain > panic_start) && (backchain < panic_end)
+			  && s390_has_cpu(bt)){
+			stack = panic_stack;
+			stack_base = panic_start;
 		} else {
 			/* invalid stackframe */
 			break;
@@ -755,7 +753,7 @@ s390_print_lowcore(char* lc, struct bt_info *bt, int show_symbols)
 	char* ptr;
 	unsigned long tmp[4];
 
-	ptr = lc + MEMBER_OFFSET("_lowcore","st_status_fixed_logout");
+	ptr = lc + OFFSET(s390_lowcore_psw_save_area);
 	tmp[0]=ULONG(ptr);
 	tmp[1]=ULONG(ptr + S390_WORD_SIZE);
 
@@ -891,12 +889,12 @@ s390_get_stack_frame(struct bt_info *bt, ulong *eip, ulong *esp)
 	int r14_offset;
 	char lowcore[LOWCORE_SIZE];
 
-	if(s390_has_cpu(bt->task))
+	if(s390_has_cpu(bt))
 		s390_get_lowcore(s390_cpu_of_task(bt->task),lowcore);
 
 	/* get the stack pointer */
 	if(esp){
-		if(s390_has_cpu(bt->task)){
+		if(s390_has_cpu(bt)){
 			ksp = ULONG(lowcore + MEMBER_OFFSET("_lowcore",
 				"gpregs_save_area") + (15 * S390_WORD_SIZE));
 		} else {
@@ -914,10 +912,9 @@ s390_get_stack_frame(struct bt_info *bt, ulong *eip, ulong *esp)
 	if(!eip)
 		return;
 
-	if(s390_has_cpu(bt->task) && esp){
-		*eip = ULONG(lowcore + MEMBER_OFFSET("_lowcore",
-		       "st_status_fixed_logout") + S390_WORD_SIZE)
-       		       & S390_ADDR_MASK;
+	if(s390_has_cpu(bt) && esp){
+		*eip = ULONG(lowcore + OFFSET(s390_lowcore_psw_save_area) +
+			S390_WORD_SIZE) & S390_ADDR_MASK;
 	} else {
 		if(!STRUCT_EXISTS("stack_frame")){
 			r14_offset = 56;
@@ -1004,7 +1001,7 @@ s390_dis_filter(ulong vaddr, char *inbuf)
 int
 s390_get_smp_cpus(void)
 {
-	return get_cpus_online();
+	return MAX(get_cpus_online(), get_highest_cpu_online()+1);
 }
 
 /*
